/*
 * Copyright (c) 2018 J. Lewis Muir <jlmuir@imca-cat.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.imca_cat.prgm;

/**
 * Indicates an error during a {@linkplain StandardIo standard I/O} operation.
 */
public final class StandardIoException extends RuntimeException {
  private static final long serialVersionUID = 1L;

  private final StandardIo io;

  /**
   * Constructs an instance with the specified type of I/O.
   *
   * @param io the type of I/O on which the error occurred
   */
  public StandardIoException(StandardIo io) {
    super("I/O type: " + io);
    this.io = io;
  }

  /**
   * Constructs an instance with the specified type of I/O and cause.
   *
   * @param io the type of I/O on which the error occurred
   * @param cause the {@linkplain Throwable#getCause() cause} of this
   *          exception; may be {@code null}
   */
  public StandardIoException(StandardIo io, Throwable cause) {
    super("I/O type: " + io, cause);
    this.io = io;
  }

  /**
   * Returns the type of I/O on which the error occurred.
   *
   * @return the I/O type
   */
  public StandardIo io() {
    return io;
  }
}
