/*
 * Copyright (c) 2018 J. Lewis Muir <jlmuir@imca-cat.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.imca_cat.prgm.internal;

import java.io.IOException;
import java.io.Writer;

/**
 * A stream that does nothing.
 * <p>
 * Anything written to this stream is discarded. All methods are no-ops.
 * <p>
 * Closing this stream has no effect. The methods in this class can be called
 * after the stream has been closed without generating an {@link IOException}.
 */
public final class NullWriter extends Writer {
  @Override
  public void write(int c) throws IOException {
    /* NO-OP */
  }

  @Override
  public void write(char[] cbuf) throws IOException {
    /* NO-OP */
  }

  @Override
  public void write(char[] cbuf, int off, int len) throws IOException {
    /* NO-OP */
  }

  @Override
  public void write(String str) throws IOException {
    /* NO-OP */
  }

  @Override
  public void write(String str, int off, int len) throws IOException {
    /* NO-OP */
  }

  @Override
  public Writer append(CharSequence csq) throws IOException {
    /* NO-OP */
    return this;
  }

  @Override
  public Writer append(CharSequence csq, int start, int end) throws IOException {
    /* NO-OP */
    return this;
  }

  @Override
  public Writer append(char c) throws IOException {
    /* NO-OP */
    return this;
  }

  @Override
  public void flush() throws IOException {
    /* NO-OP */
  }

  @Override
  public void close() throws IOException {
    /* NO-OP */
  }
}
