/*
 * Copyright (c) 2018 J. Lewis Muir <jlmuir@imca-cat.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.imca_cat.prgm.internal;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.Flushable;

/**
 * Static utility methods.
 */
public final class Utilities {
  private static final int DEFAULT_IO_BUFFER_SIZE = computeDefaultIOBufferSize();

  private Utilities() {
    throw new AssertionError();
  }

  public static int defaultIOBufferSizeInBytes() {
    return DEFAULT_IO_BUFFER_SIZE;
  }

  public static void flushSilently(Flushable f) {
    if (f == null) return;
    try {
      f.flush();
    } catch (ThreadDeath th) {
      throw th;
    } catch (Throwable th) {
      /* IGNORED */
    }
  }

  private static int computeDefaultIOBufferSize() {
    return (new BufferedOutputStream(new ByteArrayOutputStream(0)) {
      public int bufferSize() {
        return buf.length;
      }
    }).bufferSize();
  }
}
