/*
 * Copyright (c) 2018 J. Lewis Muir <jlmuir@imca-cat.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.imca_cat.prgm.internal;

import java.util.Objects;

/**
 * An object that holds a value.
 */
public final class ValueHolder<T> {
  private T value;
  private boolean set = false;

  public ValueHolder() {
    super();
  }

  public ValueHolder(T value) {
    super();
    this.value = value;
    set = true;
  }

  public T value() {
    if (!set) {
      throw new IllegalStateException("No value");
    }
    return value;
  }

  public void value(T value) {
    this.value = value;
    set = true;
  }

  public boolean hasValue() {
    return set;
  }

  public void clear() {
    value = null;
    set = false;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (!(o instanceof ValueHolder)) return false;
    ValueHolder<?> h = (ValueHolder<?>)o;
    if (set != h.set) return false;
    return Objects.equals(value, h.value);
  }

  @Override
  public int hashCode() {
    int result = (value == null) ? 0 : value.hashCode();
    result = (31 * result) + (set ? 1 : 0);
    return result;
  }

  @Override
  public String toString() {
    StringBuilder result = new StringBuilder();
    result.append('[');
    result.append("value=").append(value);
    result.append(", set=").append(set);
    result.append(']');
    return result.toString();
  }
}
