/*
 * Copyright (c) 2018 J. Lewis Muir <jlmuir@imca-cat.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.imca_cat.prgm.internal;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

/**
 * Tests {@link ValueHolder}.
 */
public final class ValueHolderTest {
  @Test
  public void testConstructor_noArgs() {
    ValueHolder<String> h = new ValueHolder<>();
    assertFalse(h.hasValue());
  }

  @Test
  public void testConstructor_initialValue() {
    ValueHolder<String> h = new ValueHolder<>("a");
    assertTrue(h.hasValue());
    assertEquals("a", h.value());
  }

  @Test
  public void testValue_setAndGet() {
    ValueHolder<String> h = new ValueHolder<>();
    h.value("a");
    assertTrue(h.hasValue());
    assertEquals("a", h.value());
  }

  @Test
  public void testValue_setAndGetTwice() {
    ValueHolder<String> h = new ValueHolder<>();
    h.value("a");
    h.value("b");
    assertTrue(h.hasValue());
    assertEquals("b", h.value());
  }

  @Test(expected = IllegalStateException.class)
  public void testValue_getNoValue() {
    ValueHolder<String> h = new ValueHolder<>();
    h.value();
  }

  @Test(expected = IllegalStateException.class)
  public void testValue_getNoValueAfterClear() {
    ValueHolder<String> h = new ValueHolder<>("a");
    h.clear();
    h.value();
  }

  @Test
  public void testClear() {
    ValueHolder<String> h = new ValueHolder<>();
    h.value("a");
    assertTrue(h.hasValue());
    h.clear();
    assertFalse(h.hasValue());
  }
}
